/***************************************************************************
 *   Copyright (C) 2004 by Marek Sopko                                     *
 *   avatar@host.sk                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

// hlavickove subory *******************************************************
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>

typedef struct __material {            // struktura OpenGL materialu
 GLfloat amb[4],dif[4],spe[4],amb_dif[4],emi[4];
 GLfloat shi,transp,sillum[4];
} _material;

typedef struct __light {               // struktura OpenGL svetla
  GLfloat amb[4],dif[4],spe[4];
  GLfloat pos[4],spot[3];
} _light;

typedef struct __camera {              // struktura OpenGL kamery
 GLfloat matrix[16];
 GLfloat posx,posy,posz,angx,angy,angz;
 GLfloat movx,movy,movz,rotx,roty,rotz;
 GLfloat oldx,oldy,oldz,targetx,targety,targetz;
 GLfloat speed,slide;
 GLfloat falldwn;
 GLubyte movetype;
 GLfloat viewdist;
 GLfloat FOV;
} _camera;

// trieda vlastnych premennych a funkcii pre pracu s OpenGL
class COGL {
public:
 COGL();                      // konstruktor
 virtual ~COGL();             // destruktor

 _material ogl_material[8];   // pouzite materialy
 _light ogl_light[8];         // pouzite svetla
 _camera ogl_camera[8];       // pouzite kamery

 GLuint ogl_lighting;         // zapnutie/vypnutie osvetlenia

 bool gl_initialize(GLvoid);  // inicializacia OpenGL
                              // prepinanie okno/cela obrazovka
 GLvoid gl_fullscreen(GLuint width,GLuint height,GLuint bits);
                              // funkcie pre pracu s kamerou
 GLvoid camera_init(GLuint a,GLfloat w,GLfloat h);
 GLvoid camera_update(GLuint a);
 GLvoid camera_move(GLuint a);
 GLvoid camera_rotate(GLuint a,GLfloat x,GLfloat y,GLfloat z);

 GLvoid obj_axis(GLfloat a);   // deklaracia objektov - suradnicova sustava
 GLvoid obj_cube0(GLfloat a);  // deklaracia objektov - kocka biela
 GLvoid obj_cube1(GLfloat a);  // deklaracia objektov - kocka farebna
 GLvoid obj_cube2(GLfloat a);  // deklaracia objektov - kocka texturovana
 GLvoid obj_crosshair(GLvoid); // deklaracia objektov - zameriavaci kriz

 GLuint ogl_texture[3];       // identifikatory nahravanych textur
 GLuint ogl_crosshair[1];     // identifikatory nahravanych textur - crosshair

 GLint LoadGLTextures(char *myfile);  // nahratie textur
 GLint LoadGLCrossHair(char *myfile); // nahratie textury crosshair

 GLvoid set_scr_size(GLuint a,GLuint b); // nastavenie rozmeru okna
private:
 GLuint scr_width,scr_height;
 GLfloat tx;
};

