/***************************************************************************
 *   Copyright (C) 2004 by Marek Sopko                                     *
 *   avatar@host.sk                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <stdlib.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>
// vlastne funkcie pre pracu s OpenGL
#include "OGL.h"
// vlastne funkcie pre pracu s OpenGL textom
#include "GLText.h"

/***************************************************************************
 * deklaracie premennych a konstant                                        *
 ***************************************************************************/
// sirka, vyska a farebna hlbka SDL okna
#define SCR_WIDTH  720
#define SCR_HEIGHT 486
#define SCR_BPP     32
// pozn.: v X11, VidMode nemeni rozlisenie obrazovky, vo Win32,
// ChangeDisplaySettings moze zmenit BPP

const SDL_VideoInfo *videoInfo=NULL; // SDL informacie o obrazovke
int videoFlags=0;                    // znacky pre SDL_SetVideoMode
SDL_Event event;                     // SDL obsluha udalosti
SDL_Surface *surface;                // SDL kresliaca plocha
Uint8 *keystate;                     // pole SDL stlacenych klaves
Uint8  mousestate;                   // SDL mys

GLuint ogl_frames;                   // pocitadlo FPS
GLuint t_t0,t_t;                     // docasne premenne
GLfloat t_sec,t_fps;                 // pocitadla - sekundy, pocet obrazkov
                                     // vygenerovanych a zobrazenych za
                                     // sekundu
int a=0,b=0,c=0;                     // docasne premenne

// instancia triedy COGL
COGL *m_cOGL;

GLfloat rot_x=1.0f;                  // premenne pre vykonavanie rotacie
GLfloat rot_y=1.0f;

// instancia triedy CGLText
CGLText m_cGLText;

/***************************************************************************
 * definicie funkcii aplikacie                                             *
 ***************************************************************************/
// uvolnenie alokovanych zdrojov a obnovenie povodnych nastaveni obrazovky *
void APP_quit(int returnCode)
{
 if (m_cOGL!=NULL) delete m_cOGL; // zrusenie instancie triedy COGL
 m_cOGL=NULL;                     // nastavenie m_cOGL=NULL

 SDL_Quit();                      // zrusenie SDL okna

 exit(returnCode);                // ukoncenie aplikacie
}

// modifikacia nastaveni kresliacej plochy po zmene velkosti okna **********
int WND_resize(int width,int height)
{
 // nastavenie SDL kresliacej plochy
 surface=SDL_SetVideoMode(width,height,SCR_BPP,videoFlags);
 if (!surface) {
  fprintf(stderr,"Chyba nastavenia SDL kresliacej plochy po zmene velkosti okna: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // reinicializacia nastaveni kamery po zmene velkosti okna
 if (m_cOGL!=NULL) m_cOGL->camera_init(0,(GLfloat) width,(GLfloat) height);

 // presun kurzora mysi na stred okna (2x aby bolo nastavenie vzdy korektne)
 SDL_WarpMouse(SCR_WIDTH/2,SCR_HEIGHT/2);
 SDL_WarpMouse(SCR_WIDTH/2,SCR_HEIGHT/2);

 return(0);
}

// obhospodarenie vstupu z klavesnice a mysi *******************************
void APP_input()
{
 a=0; b=0; c=0;
 // nacitanie polohy kurzora mysi
 mousestate=SDL_GetMouseState(&b,&a);
 SDL_WarpMouse(SCR_WIDTH/2,SCR_HEIGHT/2);
 a-=SCR_HEIGHT/2; b-=SCR_WIDTH/2;

 // nacitanie stlacenych klaves
 keystate=SDL_GetKeyState(NULL);

 if (keystate[SDLK_ESCAPE]) APP_quit(0); // ESC

 if (keystate[SDLK_UP]) {                // sipka hore
  if (m_cOGL->ogl_camera[0].speed<120.0f) m_cOGL->ogl_camera[0].speed+=2.0f;
 }
 if (keystate[SDLK_DOWN]) {              // sipka dole
  if (m_cOGL->ogl_camera[0].speed>-120.0f) m_cOGL->ogl_camera[0].speed-=2.0f;
 }
 if (keystate[SDLK_RIGHT]) {             // sipka doprava
  if (m_cOGL->ogl_camera[0].slide<120.0f) m_cOGL->ogl_camera[0].slide+=2.0f;
 }
 if (keystate[SDLK_LEFT]) {              // sipka dolava
  if (m_cOGL->ogl_camera[0].slide>-120.0f) m_cOGL->ogl_camera[0].slide-=2.0f;
 }

 // automaticke spomalovanie pohybu
 if (m_cOGL->ogl_camera[0].speed>0.0f) m_cOGL->ogl_camera[0].speed-=1.0f;
 if (m_cOGL->ogl_camera[0].speed<0.0f) m_cOGL->ogl_camera[0].speed+=1.0f;
 if (m_cOGL->ogl_camera[0].slide>0.0f) m_cOGL->ogl_camera[0].slide-=1.0f;
 if (m_cOGL->ogl_camera[0].slide<0.0f) m_cOGL->ogl_camera[0].slide+=1.0f;

 m_cOGL->camera_rotate(0,(GLfloat) a/4.0f,(GLfloat) b/4.0f,(GLfloat) c/4.0f);
 m_cOGL->camera_move(0);

 // aplikacia rotacie a pohybu na aktualnu kameru
 m_cOGL->camera_update(0);
}

// inicializacia SDL *******************************************************
int SDL_initialize(void)
{
 // inicializacia SDL video subsystemu
 if (SDL_Init(SDL_INIT_VIDEO)<0) {
  fprintf(stderr,"Chyba inicializacie SDL video subsystemu: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // ziskanie informacii o SDL video subsysteme
 videoInfo=SDL_GetVideoInfo();
 if (!videoInfo) {
  fprintf(stderr,"Chyba ziskania informacii o SDL video subsysteme: %s\n",SDL_GetError());
  APP_quit(1);
 }

 // nastavenie atributov SDL kresliacej plochy
 // 5 bitov pre cervenu, zelenu a modru farbu
 // nepouzijeme zasobnik priesvitnosti ani zasobnik stencil
 // 16 bitovy zasobnik hlbky
 // 1 pre pouzitie dvojiteho zasobnika pre prekreslovanie
 SDL_GL_SetAttribute(SDL_GL_RED_SIZE,5);
 SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,5);
 SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,5);
 SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE,0);
 SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE,0);
 SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE,16);
 SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);
 // pozn.: SDL_DOUBLEBUFFER sa pouziva v SDL_GL_SetAttribute,
 // v SDL_SetVideoMode to ovplivni iba 2D prekreslovanie

 // znacky pre SDL_SetVideoMode
 videoFlags =SDL_OPENGL;          // zapne OpenGL v SDL
 videoFlags|=SDL_GL_DOUBLEBUFFER; // alokuje dvojity zasobnik pre prekreslovanie
 videoFlags|=SDL_HWPALETTE;       // ulozi paletu do HW
 videoFlags|=SDL_RESIZABLE;       // umozni menit velkost SDL okna

 // kontrola ci mozno SDL kresliacu plochu umiestnit do HW
 if (videoInfo->hw_available) videoFlags|=SDL_HWSURFACE;
 else videoFlags|=SDL_SWSURFACE;

 // kontrola, ci mozno pracovat s HW blit
 if (videoInfo->blit_hw) videoFlags|=SDL_HWACCEL;

 // nastavenie SDL kresliacej plochy
 surface=SDL_SetVideoMode(SCR_WIDTH,SCR_HEIGHT,SCR_BPP,videoFlags);
 if (!surface) {
  fprintf(stderr,"Chyba nastavenia SDL kresliacej plochy: %s\n",SDL_GetError());
  APP_quit(1);
 }
 SDL_WM_SetCaption("1st OpenGL","1st OpenGL"); // nastavenie nazvu okna, ikony

 return(0);
}

// funkcia pre kreslenie OpenGL sceny **************************************
int draw_gl(GLvoid)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 glMultMatrixf(m_cOGL->ogl_camera[0].matrix);
 glTranslatef(m_cOGL->ogl_camera[0].posx,
              m_cOGL->ogl_camera[0].posy,
              m_cOGL->ogl_camera[0].posz);

 // definicia objektu-suradnicova sustava
 m_cOGL->obj_axis(1.0f);

 // ulozenie aktualnej matice do zasobnika matic
 glPushMatrix();

 // transformacia objektov sceny
 glTranslatef(5.0f,0.0f,0.0f);
 glRotatef(rot_x,1.0f,0.0f,0.0f);
 glRotatef(rot_y,0.0f,1.0f,0.0f);

 // definicia objektu-kocky
 m_cOGL->obj_cube1(2.0f);

 // vyber aktualnej matice zo zasobnika matic
 glPopMatrix();
 glPushMatrix();

 // transformacia objektov sceny
 glTranslatef(-5.0f,0.0f,0.0f);
 glRotatef(-rot_x,1.0f,0.0f,0.0f);
 glRotatef(-rot_y,0.0f,1.0f,0.0f);

 // definicia objektu-kocky
 m_cOGL->obj_cube1(2.0f);

 // vyber aktualnej matice zo zasobnika matic
 glPopMatrix();
 glPushMatrix();

 // transformacia objektov sceny
 glTranslatef(-20.0f,0.0f,0.0f);
 glRotatef(-rot_x,1.0f,0.0f,0.0f);
 glRotatef(-rot_y,0.0f,1.0f,0.0f);

 // pouzitie textury pre pokrytie objektu
 glBindTexture(GL_TEXTURE_2D,m_cOGL->ogl_texture[0]);
 // definicia objektu-kocky
 m_cOGL->obj_cube2(2.0f);

 // vyber aktualnej matice zo zasobnika matic
 glPopMatrix();

 // vykreslenie zameriavacieho kriza
 m_cOGL->obj_crosshair();

 // vypis textu - poloha kamery
 sprintf(m_cGLText.mybuf,"x: %6.2f",-m_cOGL->ogl_camera[0].posx);
 m_cGLText.DrawString(10.0f,30.0f);
 sprintf(m_cGLText.mybuf,"y: %6.2f",-m_cOGL->ogl_camera[0].posy);
 m_cGLText.DrawString(10.0f,20.0f);
 sprintf(m_cGLText.mybuf,"z: %6.2f",-m_cOGL->ogl_camera[0].posz);
 m_cGLText.DrawString(10.0f,10.0f);

 // pocitadlo FPS
 ogl_frames++;
 t_t=SDL_GetTicks();
 if (t_t-t_t0>=500) {
  t_sec=(t_t-t_t0)/1000.0f;
  t_fps=ogl_frames/t_sec;
  t_t0=t_t;
  ogl_frames=0;
 }
 // vypis texty - pocet obrazkov za sekundu
 sprintf(m_cGLText.mybuf,"%i fps",(GLuint) t_fps);
 m_cGLText.DrawString(10.0f,(GLfloat) SCR_HEIGHT-20.0f);

 // vymena zobrazovacich zasobnikov, zobrazenie sceny
 SDL_GL_SwapBuffers();

 rot_x+=0.5f; rot_y+=0.5f;

 return(1);
}

// hlavna vykonavacia funkcia aplikacie ************************************
int main(int argc,char **argv)
{
 int done=0;                       // premenna hlavneho cyklu
 int isActive=1;                   // SDL okno je/nie je aktivne

 m_cOGL=NULL;                      // tvorba instancie triedy COGL

 if (m_cOGL==NULL) {
  m_cOGL=new COGL();
  if (m_cOGL==NULL) {
   fprintf(stderr,"Chyba tvorby instancie triedy COGL!\n");
   APP_quit(1);
  }
 }

 SDL_initialize();                 // inicializacia SDL

 m_cOGL->gl_initialize();          // inicializacia OpenGL

 // nahratie fontu
 if (!fontLoad("data/font.tga")) {
  fprintf(stderr,"Chyba nahratia fontu!\n");
  APP_quit(1);
 }
 m_cGLText.LabelColor[0]=1.0f;
 m_cGLText.LabelColor[1]=1.0f;
 m_cGLText.LabelColor[2]=1.0f;
 m_cGLText.LabelColor[3]=1.0f;

 // nahratie textur
 if (!m_cOGL->LoadGLTextures("data/crate.bmp")) {
  fprintf(stderr,"Chyba nahratia textur!\n");
  APP_quit(1);
 }
 // nahratie crosshair
 if (!m_cOGL->LoadGLCrossHair("data/crosshair0.bmp")) {
  fprintf(stderr,"Chyba nahratia crosshair!\n");
  APP_quit(1);
 }

 WND_resize(SCR_WIDTH,SCR_HEIGHT); // upravenie velkosti SDL okna

 SDL_ShowCursor(SDL_DISABLE);

 ogl_frames=0;                     // zaciatok pocitania FPS
 t_t0=0;

 // hlavny programovy cyklus
 while (!done) {
  // ulozenie a spracuvanie udalosti
  while (SDL_PollEvent(&event)) {
   switch (event.type) {
    case SDL_ACTIVEEVENT:          // kreslenie iba ak je okno aktivne
     if (event.active.gain==0) isActive=0;
     else isActive=1;
     break;
    case SDL_VIDEORESIZE:          // zmena velkosti okna
     WND_resize(event.resize.w,event.resize.h);
     break;
    case SDL_QUIT: done=1;         // zrusenie SDL okna
     break;
    default: break;
   }
  }

  // obsluha klavesnice a mysi
  APP_input();

  // kreslenie sceny
  if (isActive) draw_gl();
 }

 // ukoncenie aplikacie
 APP_quit(0);

 return(0);
}

